@@ -3,10 +3,13 @@
 #' @description 
 #' This function helps to extract dates from a vector of files.
 #' 
-#' @param files \code{character} vector of filenames from which to extract 
-#' dates.
-#' @param pos1 \code{integer}, start of date string in \code{files}.
-#' @param pos2 \code{integer}, end of date string. 
+#' @param files A \code{character} vector of filenames from which to extract 
+#' dates. Alternatively, a \code{Raster*} with date information in its layer 
+#' \code{\link[raster]{names}}.
+#' @param pos1,pos2 Start and end of date string in \code{files} as 
+#' \code{integer}. If missing, positions are tried to be retrieved from a 
+#' look-up table provided that 'files' comply with the MODIS standard naming 
+#' convention.
 #' @param asDate \code{logical}. If \code{TRUE}, the result is converted to a 
 #' \code{Date} object.
 #' @param format \code{character}, date format. Used only if \code{asDate = TRUE}. 
@@ -22,29 +25,37 @@
 #' Matteo Mattiuzzi
 #' 
 #' @examples 
-#' \dontrun{
 #' # example on HDF files
 #' files <- c("MOD13Q1.A2010209.h18v03.005.2010239071130.hdf",
 #'            "MOD13Q1.A2010225.h18v03.005.2010254043849.hdf")
 #' extractDate(files)
-#' extractDate(files,asDate=TRUE)
+#' extractDate(files, asDate = TRUE)
 #' 
 #' # on any other file 
-#' files <- c("Myfile_20010101.XXX","Myfile_20010115.XXX","Myfile_20010204.XXX")
-#' extractDate(files,pos1=8,pos2=15)
-#' extractDate(files,pos1=8,pos2=15,asDate=TRUE,format="\%Y\%m\%d")
-#' }
+#' files <- c("Myfile_20010101.XXX", "Myfile_20010115.XXX", "Myfile_20010204.XXX")
+#' extractDate(files, pos1 = 8, pos2 = 15)
+#' extractDate(files, pos1 = 8, pos2 = 15, asDate = TRUE, format = "%Y%m%d")
 #'  
 #' @export extractDate
 #' @name extractDate
-extractDate <- function(files,pos1=10,pos2=16,asDate=FALSE,format="%Y%j")
+extractDate <- function(files, pos1, pos2, asDate = FALSE, format = "%Y%j")
 {
-  if(inherits(files,"Raster"))
-  {
+  if (inherits(files, "Raster")) {
     files <- names(files)
   }
+  
   files <- basename(files)
-  date  <- sapply(files,function(x){substr(x,pos1,pos2)})
+  
+  ## if any position indication is missing, try to retrieve it from look-up table
+  if (any(missing(pos1), missing(pos2))) {
+    ids = positionIndication(files)
+    pos1 = ids[[1]]; pos2 = ids[[2]]
+  }
+
+  date  <- sapply(files,function(x){
+    substr(x, pos1, pos2)
+  })
+  
   if(asDate)
   {
     date <- as.Date(date, format=format)
